# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import argparse
import logging
from pathlib import Path

from abcreate.bundle.errors import BundleValidationError
from abcreate.util.cli import setup_cli
from abcreate.util.log import logstats, setup_logging

log = logging.getLogger("main")


def main() -> None:
    parser = argparse.ArgumentParser(description="create an application bundle")
    setup_cli(parser)
    args = parser.parse_args()
    setup_logging(Path("abcreate.log"))

    try:
        args.func(args)
    except AttributeError:
        parser.print_usage()
        exit(1)
    except BundleValidationError as e:
        log.critical(e)
        exit(2)

    log.info(f"finished with {logstats.warnings} warnings and {logstats.errors} errors")

    if logstats.errors:
        exit(1)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import List, Optional

from pydantic_xml import BaseXmlModel, element

from .errors import BundleValidationError
from .executable import Executable
from .plist import Plist
from .symlink import Symlink

log = logging.getLogger("executable")


class Executables(BaseXmlModel):
    executables: List[Executable] = element(tag="executable")
    symlinks: Optional[List[Symlink]] = element(tag="symlink", default=list())

    @property
    def main_executable(self) -> Executable:
        try:
            return self.executables[0]
        except IndexError:
            raise BundleValidationError("no executables specified")

    def install(self, bundle_dir: Path, install_prefix: Path):
        for executable in self.executables:
            executable.install(bundle_dir, install_prefix)

        Plist().CFBundleExecutable = self.main_executable.target_name

        for symlink in self.symlinks:
            symlink.install(bundle_dir)



# SPDX-FileCopyrightText: 2026 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

try:
    from ._version import version as VERSION
except ImportError:
    VERSION = "0.0.0"



# SPDX-FileCopyrightText: 2026 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import argparse
import logging
from enum import Enum
from pathlib import Path

from abcreate.bundle import Bundle
from abcreate.util.version import VERSION

log = logging.getLogger("main")


class Command(Enum):
    CREATE = "create"


def create_app(args: argparse.Namespace) -> None:
    log.info(f"abcreate {VERSION}")
    xml_doc = args.file.read_text()
    bundle = Bundle.from_xml(xml_doc)
    bundle.create(args.output_dir, args.install_prefix)


def setup_cli(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("--version", action="version", version=f"abcreate {VERSION}")
    commands = parser.add_subparsers(help="available commands", dest="command")

    create_command = commands.add_parser(
        Command.CREATE.value, help="create application bundle"
    )
    create_command.add_argument("file", type=Path, help="XML configuration file")
    create_command.add_argument(
        "-i",
        "--install_prefix",
        type=Path,
        required=True,
        help="install prefix of the application",
    )
    create_command.add_argument(
        "-o",
        "--output_dir",
        type=Path,
        required=True,
        help="directory to create the .app bundle in",
    )
    create_command.set_defaults(func=create_app)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from .gdkpixbuf import GdkPixbuf
from .gir import Gir
from .glib import Glib
from .gtk3 import Gtk3
from .gtk4 import Gtk4

__all__ = ["GdkPixbuf", "Gir", "Glib", "Gtk3", "Gtk4"]



# SPDX-FileCopyrightText: 2026 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from .bundlevalidationerror import BundleValidationError

__all__ = ["BundleValidationError"]
